% This function compute the geodesic-intensity histogrm (GIH) as described in our paper
% 		Deformation Invariant Image Matching, H. Ling and D.W. Jacobs, 
% 		IEEE International Conference on Computer Vision (ICCV) Vol. II, pp. 1466-1473, 2005.
% 
% The function is defined as 
% [Hs, T]	= GIH(	im, ...			% input intensity image, value in [0..1]
% 					x, ...			% x of the interest point
% 					y, ...			% y of the interest point
% 					alpha, ...		% aspect ratio, in (0..1)
% 					n_color, ...	% number of color (intensity) bin 
% 					n_geo, ...		% number of geodesic distance bin
% 					geo_range, ...	% geodesic range of the support region
% 					geo_reso, ...	% sampling gap (used by contour extraction)
% 					msk, ... 		% foreground mask
% 					A, ...			% lighting coeffecients, affine lighting model I=aI+b
% 					ifig ...		% if >0, demonstrate computation
% 				)
% 
%	It compute GIH at of image 'im' at point 'x,y', the computation can be restricted within
%	 foreground defined by mask 'msk'.
%	
%	Output:
%		Hs		: each COLUMN of Hs is a vector storing the 2D geodesic-intensity histogram, 
%					with size  n_color X n_geo. I.e. the following command can be used to get
%					a 2D GIH from the i-th GIH:
%						h	= reshape(Hs(:,i), n_color, n_geo);
%					The number of columns is determined by how many different lighting allowed,
%					which is determined by 'A' as described below. 
%		T		: the geodesic distances from (x,y) to all other points within the support region
%					determined by 'geo_range'.
%	
%	Notes for inputs:
%		'im,x,y': required parameters, other ones are optimal. See the description of usage 
% 					below for details.
%		'A' 	: is an array to dealing with affine lighting change. We assume affine lighting
%					model, that is, the change of lighting is determined by fomula
%						a*I+b
%					As a result, 'A' is an array of differnt 'a', by default 
%						A = [0.95, 1, 1.05];
%					This means three GIHs are built at [x,y], therefore, 'Hs' has three columns.
%					As a special case, A=1 assumes there is no much lighting change.
%
% 
% Current, the cuntion support following usage
% 	the simplest case
% 				[Hs,T]	= GIH(im, x, y)
% 	four inputs
% 				[Hs,T]	= GIH(im, x, y, alpha)
% 	five inputs
% 				[Hs,T]	= GIH(im, x, y, n_color, n_geo)
% 	six inputs
% 				[Hs,T]	= GIH(im, x, y, alpha, n_color, n_geo)
% 	ten inputs (except 'ifig')
% 				[Hs,T]	= GIH(im, x, y, alpha, n_color, n_geo, geo_range, msk, A)
%
%
% Any questions and comments are welcome to 
% 			Haibin Ling 	(hbling AT umiacs.umd.edu)
% 			Computer Science Dept., Univ. of Maryland
% 			Mar, 13, 2006
% 


function [Hs, T]	= GIH(	im, ...			% input intensity image, value in [0..1]
							x, ...			% x of the interest point
							y, ...			% y of the interest point
							alpha, ...		% aspect ratio, in (0..1)
							n_color, ...	% number of color (intensity) bin 
							n_geo, ...		% number of geodesic distance bin
							geo_range, ...	% geodesic range of the support region
							geo_reso, ...	% sampling gap (used by contour extraction)
							msk, ... 		% foreground mask
							A, ...			% lighting coeffecients, affine lighting model I=aI+b
							ifig ...		% if >0, demonstrate computation
						)
						
%- parsing input argument
[nR,nC,nD]	= size(im);
if nD>1
	disp('Color image converted to gray image');
	im	= double(rgb2gray(im))/255;
end


%- default values
DFT_alpha		= .98;		% aspect ratio, in (0..1)
DFT_n_color		= 11;		% number of color (intensity) bin 
DFT_n_geo		= 8;		% number of geodesic distance bin
DFT_geo_range	= 1.6;		% geodesic range of the support region
DFT_A			= [.95, 1, 1.05];	% lighting coeffecients, affine lighting model I=aI+b
DFT_ifig		= -1;				% if >0, demonstrate computation


%-----------------------------------------------------------------
if nargin==3
	% the simplest case
	%			[Hs,T]	= GIH(im, x, y)
	alpha		= DFT_alpha;		% aspect ratio, in (0..1)
	n_color		= DFT_n_color;		% number of color (intensity) bin 
	n_geo		= DFT_n_geo;		% number of geodesic distance bin
	geo_range	= DFT_geo_range;	% geodesic range of the support region	
	d_geo		= geo_range/n_geo;	% geodesic distance gap
	geo_reso	= d_geo/8;			% sampling gap (used by contour extraction)
	msk			= ones(size(im));	% foreground mask
	A			= DFT_A;			% lighting coeffecients, affine lighting model I=aI+b
	ifig		= DFT_ifig;			% if >0, demonstrate computation
	
elseif nargin==4
	% four inputs
	%			[Hs,T]	= GIH(im, x, y, alpha)
	n_color		= DFT_n_color;		% number of color (intensity) bin 
	n_geo		= DFT_n_geo;		% number of geodesic distance bin
	geo_range	= DFT_geo_range;	% geodesic range of the support region	
	d_geo		= geo_range/n_geo;	% geodesic distance gap
	geo_reso	= d_geo/8;			% sampling gap (used by contour extraction)
	msk			= ones(size(im));	% foreground mask
	A			= DFT_A;			% lighting coeffecients, affine lighting model I=aI+b
	ifig		= DFT_ifig;			% if >0, demonstrate computation
	
elseif nargin==5
	% five inputs
	%			[Hs,T]	= GIH(im, x, y, n_color, n_geo)
	alpha		= DFT_alpha;		% aspect ratio, in (0..1)
	geo_range	= DFT_geo_range;	% geodesic range of the support region	
	d_geo		= geo_range/n_geo;	% geodesic distance gap
	geo_reso	= d_geo/8;			% sampling gap (used by contour extraction)
	msk			= ones(size(im));	% foreground mask
	A			= DFT_A;			% lighting coeffecients, affine lighting model I=aI+b
	ifig		= DFT_ifig;			% if >0, demonstrate computation
	
elseif nargin==6
	% six inputs
	%			[Hs,T]	= GIH(im, x, y, alpha, n_color, n_geo)
	geo_range	= DFT_geo_range;	% geodesic range of the support region	
	d_geo		= geo_range/n_geo;	% geodesic distance gap
	geo_reso	= d_geo/8;			% sampling gap (used by contour extraction)
	msk			= ones(size(im));	% foreground mask
	A			= DFT_A;			% lighting coeffecients, affine lighting model I=aI+b
	ifig		= DFT_ifig;			% if >0, demonstrate computation
	
elseif nargin==10
	ifig		= DFT_ifig;			% if >0, demonstrate computation
	
elseif nargin~=11
	disp('Not enough input parameter');
	help GIH;
	if exist('A')		n_A		= size(A);
	else				n_A		= 3;
	end
	Hs		= (1/n_color/n_geo)*ones(n_color*n_geo,n_A);		% each column is a GIH
	T		= zeros(size(im));
	return;
end	
	


%- change data into the sense of beta0 instead of alpha, i.e.
%		alpha	= beta0/(1+beta0);
%		beta0	= alpha/(1-alpha);

geo_range		= geo_range/(1-alpha);
geo_reso		= geo_reso/(1-alpha);
beta0			= alpha/(1-alpha);
d_geo			= geo_range/n_geo;	% geodesic distance gap

%- set boundary of mask as 0
msk([1 end],:)	= 0;
msk(:,[1 end])	= 0;


%- Call the function to compute GIH
[Hs, T]	= GIH_(	im, ...			% input intensity image, value in [0..1]
				x, ...			% x of the interest point
				y, ...			% y of the interest point
				beta0, ...		% aspect ratio, in [0..inf)
				n_color, ...	% number of color (intensity) bin 
				n_geo, ...		% number of geodesic distance bin
				d_geo, ...		% geodesic distance gap
				geo_reso, ...	% sampling gap (used by contour extraction)
				msk, ... 		% foreground mask
				A, ...			% lighting coeffecients, affine lighting model I=aI+b
				ifig ...		% if >0, demonstrate computation
			);
