% function [Hs,T]	= GH_lighting_mask(im, msk, A, x,y, ...
%  								   alpha, n_color, n_geo, d_geo, geo_reso, ifig);
% 
%	Computing the geodesic histogram of image 'im' at point 'x,y', the
%	image is restricted with boundary, which is indicated by the binary
%	matrix msk.
%	
%	Deals with lighting !!!!!!!
%		Assume affine lighting change   a*I+b
%
%	Output:
%		Hs		: each column of Hs is a vector storing the 2D geodesic-intensity histogram, 
%					with size  n_color X n_geo
%		T		: the distances from (x,y) to all other points
%	
%	Input:
% 		im, msk, x,y	: image, mask, point interested
%							  	im is grayscale valued, i.e., in [0,1]
%								image and mask MUST be of double
%		alpha				: the aspect ratio, when alpha is infinite,
%								deformation invariant is achieved
% 		max_geo			: the range of the geodesic distance for the histogram 
% 		n_geo,n_color: number of bins of the histogram
% 		ifig				: if > 0, then demo in figure(ifig)
%
%					Haibin Ling 	(hbling AT umiacs.umd.edu)
%					Computer Science Dept., Univ. of Maryland
%					Mar, 3, 2005
%

function [Hs, T]	= GIH_(	im, ...			% input intensity image, value in [0..1]
							x, ...			% x of the interest point
							y, ...			% y of the interest point
							beta0, ...		% aspect ratio, in [0..inf)
							n_color, ...	% number of color (intensity) bin 
							n_geo, ...		% number of geodesic distance bin
							d_geo, ...		% geodesic distance gap
							geo_reso, ...	% sampling gap (used by contour extraction)
							msk, ... 		% foreground mask
							A, ...			% lighting coeffecients, affine lighting model I=aI+b
							ifig ...		% if >0, demonstrate computation
						)
						
%%
%%-- Preparation
%%
% beta0	= alpha/(1-alpha);
% d_geo	= d_geo / (1-alpha);

max_geo	= d_geo*(n_geo+1);
max_a	= max(A);
min_a	= min(A);
max_geo	= max_geo*max_a;


%%
%%-- Compute the geodesic distance use fast marching
%%

%-FAST MARCHING: compute the geodesic distance from the (x,y) --------
[Fx,Fy]		= gradient(im*beta0);
F       	= (1+Fx.^2+Fy.^2);
d_ceiling	= max_geo;
T			= fast_marching_grid_C( im, [x,y], msk, F, d_ceiling );

%- dealing with boundaries
T				= min(T,max_geo+1);
T([1,end],:)	= T([2,end-1],:);
T(:,[1,end])	= T(:,[2,end-1]);

%- demo fastmarching result
if ifig>0	
	figure(ifig);	clf; 
	subplot(2,2,1);	imagesc(im); colormap(gray); hold on;
	plot(x,y,'r+');	axis equal; axis ij;
	subplot(2,2,3); imagesc(T); colormap(gray); colorbar; hold on;
	plot(x,y,'r+'); axis equal; axis ij; axis tight;
	subplot(2,2,2); imagesc(im); colormap(gray); hold on;
	plot(x,y,'bx','markersize',30); axis equal; axis ij;
	
	figure(ifig+100);	clf;	%imagesc(msk); 
	imagesc(im);	colormap(gray); hold on;
	plot(x,y,'yx','markersize',30); axis equal; axis ij;
	
	figure(ifig+200);	clf; hold on; colormap(gray);
end


%%
%%-- CONTOURC: extracting geodesic level curves, the most time consuming part!! -------
%%
ds		= (geo_reso*min_a) : (geo_reso*min_a) : max_geo;
C		= contourc(T,ds);			% get geodesic level curves
n_C		= size(C,2);


%%
%%--- Sampling along each geodesic level curves
%%
all_samples	= cell(n_C,1);		% store all the sampled position and the geodesics
iHead		= 1;
n_curv		= 0;
while iHead<n_C
	cur_geo	= C(1,iHead);						% the geodesic distance on current contour
	n_cont	= C(2,iHead);
	C_tmp	= C(:,iHead+1:iHead+C(2,iHead))';	% the contour to be dealing
	iHead	= iHead+n_cont+1;					% point to the next contour
	
	%- sampling
	if 0	[Xs,Ys,Is0] = geo_sampling_C(C_tmp,geo_reso,im,beta0);	% C++
	else	[Xs,Ys,Is0] = geo_sampling2(C_tmp,geo_reso,im,beta0);		% matlab
	end
	
	n_curv	= n_curv+1;
	all_samples{n_curv}	= [Xs,Ys, cur_geo*ones(length(Xs),1)];
	
	%- demo sampling results
	if ifig>0
		disp(['current geodesic contour value = ' num2str(cur_geo)]);
		scolors	= 'rgbcmy';
		sClr	= scolors(mod(n_curv,6)+1);
		figure(ifig);		plot(C_tmp(:,1),C_tmp(:,2),['-' sClr]);
		if length(Xs)>0		plot(Xs,Ys,['.'  sClr]);	end % the sampling
		axis ij;
		figure(ifig+100);	plot(C_tmp(:,1),C_tmp(:,2),['-' sClr]);
		if length(Xs)>0		plot(Xs,Ys,['.'  sClr]);	end % the sampling
		axis ij;
	end
end

all_samples	= all_samples(1:n_curv);
all_samples	= cell2mat(all_samples);
n_samp		= size(all_samples,1);
[nR,nC]		= size(im);
if n_samp>0
	iXs		= min(max(round(all_samples(:,1)),1),nC);
	iYs		= min(max(round(all_samples(:,2)),1),nR);
	idx		= (iXs-1)*nR+iYs;

	% remove background points
	id_fg	= find(msk(idx)>0);				
	idx		= idx(id_fg);
	Gs		= all_samples(id_fg,3);
	Is		= im(idx);
	n_samp	= length(Gs);
end


%%
%%--- Build GIHs, descetize all samples and remove background points
%%
n_A		= length(A);
Hs		= (1/n_color/n_geo)*ones(n_color*n_geo,n_A);		% each column is a GIH

if n_samp==0
	disp('Error sampling, zero sampling points!');
	return;
end


for ia=1:length(A)
	a		= A(ia);
	if 0
		% shrink the range of intensity as well as the geodesic distance
		I_reso	= 2*a/n_color;		% decide the bin resolution
		G_reso	= a*d_geo;
		iGs		= ceil(Gs/G_reso);
		iIs		= ceil((Is-im(y,x)+a)/I_reso);

	else
		% shrink the geodesic range only, then do lighting normalization
		cur_range	= max_geo*a/max_a;
		id_sel	= find(Gs<=cur_range);
		if length(id_sel)>0
			Isel	= Is(id_sel);
			miu_I	= mean(Isel);
			if length(id_sel)>1		sig_I	= std(Isel);
			else					sig_I = 1;
			end
			Inorm	= (Isel-miu_I)/sig_I;		% normalize
			I_reso	= 6/n_color;
			iIs		= min(max(ceil((Inorm+3)/I_reso),1),n_color);		

			Gsel	= Gs(id_sel);
			G_reso	= a*d_geo;
			iGs		= ceil(Gsel/G_reso);
		else
			iIs		= [];
			iGs		= [];
			disp(['no sample points!!! (in GIH(.))']);
		end
	end


	% build the histogram
	if length(iIs)>0
		if 1
			[H_count,H]	= hist_count_C (iIs,iGs,n_color,n_geo);		% C++ code
		else
			H_count	= zeros(n_color,n_geo);	% intensity-geodesic 2D histogram
			for ic=1:n_color
				for ig=1:n_geo
					H_count(ic,ig)	= H_count(ic,ig)+length(find(iIs==ic & iGs==ig));
				end
			end
			%- normalize
			H			= H_count;
			sumH		= sum(H,1);
			idx_0		= find(sumH<.5);
			H(:,idx_0)	= 1;
			sumH(idx_0)	= n_color;
			H			= H./(repmat(sumH,n_color,1));		% normalize each colomn
			H			= H/sum(H(:));							% normalize the whole histogram
		end

		Hs(:,ia)	= reshape(H,n_color*n_geo,1);
	end

	%--- demo ---
	if ifig>0
		% draw sampled points used in current GIH
		figure(ifig);
		subplot(2,2,4);	
		imagesc(H);	axis xy;

		fR	= ceil(sqrt(n_A));
		fC	= fR;
		figure(ifig+200);	subplot(fR,fC,ia);
		imagesc(H);	axis xy;
		title(['a=' num2str(a)]);
		keyboard;	
	end
end